@@ -2,11 +2,13 @@
 
 __prog__ = "misis-elibrary"
 __desc__ = "Программа для загрузки книг из электронной библиотеки МИСИС."
-__version__ = "0.0.2"
+__version__ = "0.0.3"
 
 import argparse
 import imghdr
 import sys
+from dataclasses import dataclass
+from enum import IntEnum
 from pathlib import Path
 
 import img2pdf
@@ -15,6 +17,7 @@ from bs4 import BeautifulSoup
 from requests.cookies import RequestsCookieJar
 
 BASE_URL = "http://elibrary.misis.ru/"
+SEARCH_URL = BASE_URL + "search2.php?action=process"
 
 HEADERS = {
     "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
@@ -25,6 +28,14 @@ HEADERS = {
 }
 
 
+class ExitCodes(IntEnum):
+    SUCCESS = 0
+    INVALID_ARGUMENTS = 2
+    LOGIN_FAILED = 3
+    BOOK_NOT_FOUND = 4
+    NO_BOOKS_FOUND = 5
+
+
 def soup(text: str):
     return BeautifulSoup(text, "html.parser")
 
@@ -57,19 +68,16 @@ def get_hash_url(id: int, page: int) -> str:
 
 
 def get_path(user_path: Path, metadata: dict[str, str]) -> Path:
-    if user_path.is_dir():
-        return user_path / f'{metadata["Название"]}.pdf'
-    else:
-        return user_path
+    return user_path / f'{metadata["Название"]}.pdf'
 
 
 def check_path(user_path: Path):
-    if not user_path.is_dir() and not user_path.parent.exists:
-        raise FileNotFoundError(f"No such file or directory: '{user_path}'")
+    if not user_path.is_dir():
+        raise FileNotFoundError(f"No such directory: '{user_path}'")
 
 
 def auth(
-    login: str, password: str, redirect_url: str
+    login: str, password: str, redirect_url: str = ""
 ) -> tuple[RequestsCookieJar, requests.Response]:
     """
     Функция возвращает сессию и ответ на запрос к redirect_url
@@ -88,13 +96,88 @@ def auth(
     response = requests.post(LOGIN_URL, payload, cookies=session, headers=HEADERS)
     if login_failed(response):
         print(f"Не удалось войти.", file=sys.stderr)
-        exit(3)
+        exit(ExitCodes.LOGIN_FAILED)
     return session, response
 
 
-def get_metadata(id: int, metadata_response: requests.Response | None) -> dict[str, str] | None:
+@dataclass
+class Book:
+    id: int
+    title: str
+    authors: str
+    year: int
+
+
+def search(query: str, session: RequestsCookieJar) -> list[Book]:
+    query = query.replace('"', "'")
+    payload = {
+        "txtQuery": f'(GeneralText contains "{query}")',
+        "cbQuickQuery": 1,
+        "cbQuickGeneral": 1,
+    }
+    session["__kt_batch_size"] = "1024"  # Не думаю, что кому-нибудь понадобится больше результатов
+    search_response = requests.post(SEARCH_URL, payload, cookies=session, headers=HEADERS)
+    del session["__kt_batch_size"]
+    content = soup(search_response.text).find("div", id="content")
+    table = content.find("table", class_="kt_collection")
+    if table.find("td").text == "Нет документов или папок соответствующих этому запросу.":
+        print(f"Не найдено книг по запросу '{query}'.", file=sys.stderr)
+        exit(ExitCodes.NO_BOOKS_FOUND)
+
+    search_results: list[Book] = []
+    for row in table.find("tbody").find_all("tr"):
+        columns = list(row.find_all("td"))
+        link = columns[1].find("a")
+        search_results.append(
+            Book(
+                id=int(link["href"].split("=")[-1]),
+                title=link.text,
+                authors=columns[3].text.strip(),
+                year=int(columns[4].text.strip()),
+            )
+        )
+    return search_results
+
+
+def print_search_results(search_results: list[Book]):
+    for i in range(len(search_results)):
+        book = search_results[len(search_results) - i - 1]
+        print(
+            f"{len(search_results) - i}. {book.authors}{' - ' if book.authors else ''}{book.title} ({book.year})"
+        )
+
+
+def parse_indexes(indexes_string: str, index_max: int) -> list[int]:
+    indexes = []
+    for index in indexes_string.split():
+        parts = tuple(map(lambda x: x - 1, map(int, index.split("-"))))
+        if parts[-1] >= index_max:
+            raise ValueError("Index out of range")
+        if len(parts) == 1:
+            indexes.append(parts[0])
+        elif len(parts) == 2:
+            indexes += list(range(parts[0], parts[1] + 1))
+        else:
+            raise ValueError("Can't parse indexes")
+    return indexes
+
+
+def get_search_results(search_results: list[Book]) -> list[Book]:
+    return list(
+        map(
+            lambda i: search_results[i],
+            parse_indexes(
+                input("Выберите книги для загрузки (например: 1 2 3, 1-3): "), len(search_results)
+            ),
+        )
+    )
+
+
+def get_metadata(
+    id: int, session: RequestsCookieJar, metadata_response: requests.Response | None = None
+) -> dict[str, str] | None:
     if metadata_response is None:
-        metadata_response = requests.get(get_metadata_url(id), headers=HEADERS)
+        metadata_response = requests.get(get_metadata_url(id), cookies=session, headers=HEADERS)
     content = soup(metadata_response.text).find("div", id="content")
 
     metadata: dict[str, str] = {}
@@ -126,7 +209,7 @@ def download(
 
     if first_hash_response.text != "0":
         print(f"Нет книги с ID {id}.", file=sys.stderr)
-        exit(2)
+        exit(ExitCodes.BOOK_NOT_FOUND)
 
     sys.stdout.write(f"Загружаем 1 страницу...")
     first_page = requests.get(get_page_url(id, 0), cookies=session, headers=HEADERS)
@@ -158,35 +241,62 @@ def main():
 
     parser.add_argument("-l", "--login", required=True, type=str, metavar="Логин")
     parser.add_argument("-p", "--password", required=True, type=str, metavar="Пароль")
-    parser.add_argument("-i", "--id", required=True, type=int, metavar="ID", help="ID книги")
+    parser.add_argument("-i", "--id", required=False, type=int, metavar="ID", help="ID книги")
     parser.add_argument(
-        "-o",
-        "--output-path",
+        "-d",
+        "--directory",
+        metavar="Директория",
         type=str,
         default=".",
-        metavar="Путь",
-        help="Может быть путём как к существующей директории, так и к файлу.",
+        help="Директория для загрузки книг. Если не указана, то используется текущая",
+    )
+    parser.add_argument(
+        "query",
+        metavar="Запрос",
+        type=str,
+        nargs="*",
+        help="Запрос для поиска",
     )
 
     args = parser.parse_args()
+    args.query = " ".join(args.query).strip()
 
-    path = Path(args.output_path)
-    check_path(Path(args.output_path))
+    base_path = Path(args.directory)
+    check_path(base_path)
 
-    session, metadata_response = auth(args.login, args.password, get_metadata_url(args.id))
+    ids = []
+    metadata_response = None
+    if args.query:
+        session, _ = auth(args.login, args.password)
 
-    metadata = get_metadata(args.id, metadata_response)
-    if metadata is None:
-        print(f"Нет книги с ID {id}.", file=sys.stderr)
-        exit(2)
-    print_metadata(metadata)
+        search_results = search(args.query, session)
+        print_search_results(search_results)
+        ids = map(lambda book: book.id, get_search_results(search_results))
+    elif args.id is None:
+        print("Передайте программе либо ID, либо запрос.", file=sys.stderr)
+        exit(ExitCodes.INVALID_ARGUMENTS)
+    else:
+        if args.id <= 0:
+            print("Передан некорректный ID", file=sys.stderr)
+            exit(ExitCodes.INVALID_ARGUMENTS)
 
-    path = get_path(path, metadata)
-    print(f"Загружаем книгу в '{path}'...")
+        session, metadata_response = auth(args.login, args.password, get_metadata_url(args.id))
+        ids = [args.id]
 
-    pdf = download(args.id, session)
-    with open(path, "wb") as f:
-        f.write(pdf)
+    for id in ids:
+        metadata = get_metadata(id, session, metadata_response)
+        metadata_response = None
+        if metadata is None:
+            print(f"Нет книги с ID {id}.", file=sys.stderr)
+            exit(ExitCodes.BOOK_NOT_FOUND)
+        print_metadata(metadata)
+
+        path = get_path(base_path, metadata)
+        print(f"Загружаем книгу в '{path}'...")
+
+        pdf = download(id, session)
+        with open(path, "wb") as f:
+            f.write(pdf)
 
 
 if __name__ == "__main__":
